import { AsyncAbstractDriverFactory, SyncAbstractDriverFactory, AsyncModels } from '..';
import { KeyValueModel as SyncKeyValueModel } from '../models/sync/key-value-model.class';
import { CollectionModel as SyncCollectionModel } from '../models/sync/collection-model.class';
import { KeyValueModel as AsyncKeyValueModel } from '../models/async/key-value-model.class';
import { CollectionModel as AsyncCollectionModel } from '../models/async/collection-model.class';
export declare type PushDataItem = {
    type: 'put';
    /**
     * This pushed data belongs to the model name.
     */
    modelName: string;
    /**
     * The millisecond timestamp when this data was changed.
     */
    timestampMs: number;
    /**
     * The key of the data in the model to which it belongs.
     */
    key: string;
    /**
     * The value of this data.
     */
    value: unknown;
} | {
    type: 'forget';
    /**
     * This pushed data belongs to the model name.
     */
    modelName: string;
    /**
     * The millisecond timestamp when this data was changed.
     */
    timestampMs: number;
    /**
     * The key of the data in the model to which it belongs.
     */
    key: string;
    /**
     * The value of this data, this value is always null because the intent is to delete.
     */
    value: null;
};
export declare type PushData = Array<PushDataItem>;
export declare type WaitingPushDataItem = PushDataItem & {
    /**
     * This value represents whether the data is being processed by a pushdatahandler, similar to a lock.
     */
    pushing: boolean;
};
export declare type WaitingPushData = Array<WaitingPushDataItem>;
export interface CloudSynchronizerOptionsInterface {
    disabled: boolean;
    /**
     * Synchronize these models.
     */
    models: Array<SyncCollectionModel | SyncKeyValueModel | AsyncCollectionModel | AsyncKeyValueModel>;
    /**
     * Whether to enable automatic push.
     * If true, when the data in the model is changed,
     * if no new data is changed within a certain period of time (determined by "minSyncInterval"),
     * push will be automatically executed.
     */
    autoPush: boolean;
    /**
     * Minimum model data synchronization interval (ms), recommended is 2000 ms.
     * When the data is updated, Kurimudb will debounce, to avoid sending too many requests to the server at the same time.
     * This value is the milliseconds of debounces performed.
     * Please determine the specific value according to the tolerable delay in your business.
     */
    minSyncInterval: number;
    /**
     * How often do I retry when the operation fails (ms), recommended is 3000 ms.
     */
    retryDelayInterval: number;
    /**
     * Maximum number of retries, recommended is 3.
     */
    maxRetries: number;
    /**
     * Which driver do you want to use to store the cached data generated during synchronization.
     * It's best to choose the driver your model is using.
     */
    driver: SyncAbstractDriverFactory | AsyncAbstractDriverFactory;
    /**
     *
     */
    pushDataHandler: (data: PushData) => Promise<boolean> | boolean;
    /**
     *
     */
    pullDataHandler: () => Promise<boolean> | boolean;
}
/**
 * 148 行：
 * 逻辑不太对：
 * 应该这样，不预先删除队列里的数据，而是在内存中标记这些数据是否处于同步状态，应该是模型名+键名+时间戳作为唯一
 * 然后当 push 函数被触发时，忽略内部被内存中标记为正在同步的数据
 * 当 push 完成后，再将其标记为已成功（即删除队列）
 * !!!!! 当首次执行 push 时，把 pushing 改为 false，避免上次同步因为某些原因，部分数据提交中断的问题
 *
 * 按时间排序，从最新的开始遍历
 * 把每个遍历到对象的modelname和key拼在一起存到 KMap 里
 *
 * 其他要实现的：
 * 持久化 _waitingPushData
 */
export interface CloudSynchronizerInterface {
    _waitingPushData: WaitingPushData;
}
export declare class CloudSynchronizer extends AsyncModels.keyValue<CloudSynchronizerInterface> {
    readonly cloudOptions: CloudSynchronizerOptionsInterface;
    readonly models: import("..").KMap<SyncCollectionModel<any, undefined> | SyncKeyValueModel<Record<string, any>, undefined> | AsyncCollectionModel<any, import("..").AsyncAbstractDriverInterface> | AsyncKeyValueModel<Record<string, any>, import("..").AsyncAbstractDriverInterface>>;
    constructor(cloudOptions: CloudSynchronizerOptionsInterface);
    push(_waitingPushData?: WaitingPushData, maxRetries?: number): Promise<void>;
    private _pushWaitingPushData;
    private _processWaitingPushData;
    private _processedWaitingPushData;
}
